using System;
using System.ComponentModel;

namespace NinjaTrader.Data
{
    public class SmartRenkoType : BarsType
    {
#pragma warning disable 169
        private static bool registered = Register(new SmartRenkoType());
#pragma warning restore 169
        public override void Add(Bars bars, double open, double high, double low, double close, DateTime time, long volume, bool isRealtime)
        {
            if (bars.Count < 1 || bars.Period.Value2 == 1 && bars.IsNewSession(time))
            {
                AddBar(bars, close, close, close, close, time, 0, isRealtime);
                AddBar(bars, close, close, close, close, time, volume, isRealtime);
                return;
            }

            Bar bar = (Bar)bars.Get(bars.Count - 1);
            Bar bar1 = (Bar)bars.Get(bars.Count - 2);

            double tickSize = bars.Instrument.MasterInstrument.TickSize;
            double rangeValue = Math.Floor(10000000.0 * bars.Period.Value * tickSize) / 10000000.0;

            double mymax = bar1.Close >= bar1.Open ? bar1.Close : bar1.Close + rangeValue;
            double mymin = bar1.Close <= bar1.Open ? bar1.Close : bar1.Close - rangeValue;

            if (bars.Instrument.MasterInstrument.Compare(close, mymax + rangeValue) >= 0)
            {
                bars.RemoveLastBar();
                AddBar(bars, bar.Open, mymax + rangeValue, bar.Low, mymax + rangeValue, time, bar.Volume, isRealtime);
                bool isLastNewBar = close < mymax + 2 * rangeValue;
                double newBarOpen = mymax + rangeValue;
                double newClose = Math.Min(mymax + 2 * rangeValue, close);

                do
                {
                    AddBar(bars, newBarOpen, newClose, newBarOpen, newClose, time, isLastNewBar ? volume : 0, isRealtime);
                    newBarOpen = newClose;
                    newClose = Math.Min(newClose + rangeValue, close);
                    isLastNewBar = close == newClose;
                }
                while (bars.Instrument.MasterInstrument.Compare(close, newClose) > 0);
            }
            else
                if (bars.Instrument.MasterInstrument.Compare(mymin - rangeValue, close) >= 0)
                {
                    bars.RemoveLastBar();
                    AddBar(bars, bar.Open, bar.High, mymin - rangeValue, mymin - rangeValue, time, bar.Volume, isRealtime);
                    double newClose = Math.Max(mymin - 2 * rangeValue, close);
                    double newBarOpen = mymin - rangeValue;
                    bool isLastNewBar = close > mymin - 2 * rangeValue;
                    do
                    {
                        AddBar(bars, newBarOpen, newBarOpen, newClose, newClose, time, isLastNewBar ? volume : 0, isRealtime);
                        newBarOpen = newClose;
                        newClose = Math.Max(newClose - rangeValue, close);
                        isLastNewBar = close == newClose;
                    }
                    while (bars.Instrument.MasterInstrument.Compare(newClose, close) > 0);
                }
                else
                    UpdateBar(bars, open, high, low, close, time, volume, isRealtime);
        }

        public override PeriodType BuiltFrom
        {
            get { return PeriodType.Tick; }
        }

        public override Gui.Chart.ChartStyleType[] ChartStyleTypesSupported
        {
            get { return new Gui.Chart.ChartStyleType[] { Gui.Chart.ChartStyleType.Custom6, Gui.Chart.ChartStyleType.CandleStick }; }
        }

        public override int GetInitialLookBackDays(Period period, int barsBack)
        {
            return 1;
        }

        public override void ApplyDefaults(Gui.Chart.BarsData barsData)
        {
            barsData.DaysBack = 3;
            barsData.Period.Value = 4;
        }

        public override string ChartDataBoxDate(DateTime time)
        {
            return time.ToString(Cbi.Globals.CurrentCulture.DateTimeFormat.ShortDatePattern);
        }

        public override string ChartLabel(Gui.Chart.ChartControl chartControl, DateTime time)
        {
            return time.ToString(chartControl.LabelFormatTick, Cbi.Globals.CurrentCulture);
        }

        public override object Clone()
        {
            return new SmartRenkoType();
        }

        public override int DefaultValue
        {
            get { return 4; }
        }

        public override string DisplayName
        {
            get { return "SmartRenko"; }
        }

        public override PropertyDescriptorCollection GetProperties(PropertyDescriptor propertyDescriptor, Period period, Attribute[] attributes)
        {
            PropertyDescriptorCollection properties = base.GetProperties(propertyDescriptor, period, attributes);

            properties.Remove(properties.Find("BasePeriodType", true));
            properties.Remove(properties.Find("BasePeriodValue", true));
            properties.Remove(properties.Find("PointAndFigurePriceType", true));
            properties.Remove(properties.Find("ReversalType", true));

            Gui.Design.DisplayNameAttribute.SetDisplayName(properties, "Value", "\rBrick size");
            Gui.Design.DisplayNameAttribute.SetDisplayName(properties, "Value2", "\rNew Session Reset(1=YES)");

            return properties;
        }
        public override bool IsIntraday
        {
            get { return true; }
        }

        public override double GetPercentComplete(Bars bars, DateTime now)
        {
            throw new ApplicationException("GetPercentComplete not supported in " + DisplayName);
        }

        public SmartRenkoType() : base(PeriodType.Custom6) {}

        public override string ToString(Period period)
        {
            return String.Format("SmartRenko {0} Ticks", period.Value);
        }
    }
}